header {
/*
 [The "BSD licence"]
 Copyright (c) 2003 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using Antlr.StringTemplate;
using System.Collections;
using System.IO;
}

options {
	language="CSharp";
	namespace="Antlr.StringTemplate.Language";
}

class StringTemplateLanguageEvaluator extends TreeParser;

options {
    importVocab=StringTemplateLanguageParser;
    ASTLabelType = "StringTemplateAST";
}

{
    public class NameValuePair {
        public String name;
        public Object value;
    };

    protected StringTemplate self = null;
    protected TextWriter outWriter = null;
    protected ASTExpr chunk = null;

    /** Create an evaluator using attributes from self */
    public StringTemplateLanguageEvaluator(StringTemplate self, ASTExpr chunk, TextWriter outWriter) {
        this.self = self;
        this.chunk = chunk;
        this.outWriter = outWriter;
    }

}

action
{
    Object e=null;
}
    :   e=expr {chunk.WriteAttribute(self, e, outWriter);}
    ;

expr returns [Object value=null]
{
    Object a=null, b=null, e=null;
    Hashtable argumentContext=null;
}
    :   #(PLUS a=expr b=expr {value = chunk.Add(a, b);})
    |   value=templateApplication
    |   value=attribute
    |   value=templateInclude
    |   #(VALUE e=expr)
        // convert to string (force early eval)
        {
        StringWriter buf = new StringWriter();
        chunk.WriteAttribute(self, e,buf);
        value = buf.ToString();
        }
    ;

templateInclude returns [Object value=null]
{
    Hashtable argumentContext = null;
    String name = null;
    Object n=null;
}
    :   // for includes, evaluating arg list now is ok
        #(  INCLUDE
            (   t2:ID argumentContext=argList[null]
                {name=t2.getText();}
/*
            |   #( VALUE n=expr argumentContext=argList[null] )
                {name=n.ToString();}
*/
            )
         )
        {value = chunk.GetTemplateText(self, name, argumentContext);}
    ;

/** Apply template(s) to an attribute; can be applied to another apply
 *  result.  I am not too efficient here as I write to a StringWriter and
 *  then dump for the common <attribute:template()> case, but I'll fix later.
 */
templateApplication returns [Object value=null]
{
Object a=null;
ArrayList templatesToApply=new ArrayList();
}
    :   #(  APPLY a=expr (template[templatesToApply])+
            {value = chunk.ApplyListOfAlternatingTemplates(self, a,templatesToApply);}
         )
    ;

template[ArrayList templatesToApply]
{
Hashtable argumentContext = null;
Object n = null;
}
    :   #(  TEMPLATE
            (   t:ID args:. // don't eval argList now; must re-eval each iteration
                {
                String templateName = t.getText();
                StringTemplateGroup group = self.Group;
                StringTemplate embedded = group.GetEmbeddedInstanceOf(self, templateName);
                if ( embedded != null ) {
                    embedded.ArgumentsAST = #args;
                    templatesToApply.Add(embedded);
                }
                }

            |	anon:ANONYMOUS_TEMPLATE
                {
                StringTemplate anonymous = anon.StringTemplate;
                //anonymous.setArgumentContext(new Hashtable());
                templatesToApply.Add(anonymous);
                }

/*
            |   #( VALUE n=expr argumentContext=argList[null] )
                {
                String templateName = n.ToString();
                StringTemplateGroup group = self.Group;
                StringTemplate embedded = group.GetEmbeddedInstanceOf(self, templateName);
                if ( embedded != null ) {
                    embedded.ArgumentsAST#args);
                    templatesToApply.Add(embedded);
                }
                }
*/
            )
         )
    ;

ifCondition returns [bool value=false]
{
    Object a=null, b=null;
}
    :   a=ifAtom {value = a != null;}
    |   #(NOT a=ifAtom) {value = a == null;}
	;

ifAtom returns [Object value=null]
    :   value=expr
    ;

attribute returns [Object value=null]
{
    Object obj = null;
}
    :   #( DOT obj=attribute prop:ID )
        {value = chunk.GetObjectProperty(self, obj, prop.getText());}

    |   i3:ID {value=self.GetAttribute(i3.getText());}

    |   i:INT {value = Int32.Parse(i.getText());}

    |   s:STRING {value=s.getText();}
    ;

argList[Hashtable initialContext]
    returns [Hashtable argumentContext=null]
{
    argumentContext = initialContext;
    if ( argumentContext == null ) {
        argumentContext=new Hashtable();
    }
}
    :   #( ARGS (argumentAssignment[argumentContext])* )
	;

argumentAssignment[Hashtable argumentContext]
{
    Object e = null;
}
	:	#( ASSIGN arg:ID e=expr {if ( e != null ) argumentContext[arg.getText()] = e;} )
	;

